//--------------------------------------------------------------
// File     : stm32_ub_din_dma.c
// Datum    : 07.06.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO, TIM, DMA, MISC
// Funktion : Multi GPIO-Input per DMA2 und Timer1
//
// Hinweis  : Je nach Einstellung entweder 8 oder 16
//            Digital-Input-Kanle
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_din_dma.h"



//--------------------------------------------------------------
// Globale Variable
//--------------------------------------------------------------
DIN_DMA_STATUS_t din_dma_status=DIN_DMA_READY;


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_DIN_DMA_InitIO(void);
void P_DIN_DMA_InitTIM(void);
void P_DIN_DMA_InitDMA(void);
void P_DIN_DMA_InitNVIC(void);
void P_DIN_DMA_start(void);



//--------------------------------------------------------------
// Init aller Input Kanaele
//--------------------------------------------------------------
void UB_DIN_DMA_Init(void)
{
  uint16_t n;

  // init vom Puffer
  for(n=0;n<DIN_DMA_LENGTH;n++) {
    DIN_DMA_PUFFER[n]=0;
  }

  // init der GPIOs
  P_DIN_DMA_InitIO();
  // init vom Timer
  P_DIN_DMA_InitTIM();
  // init vom DMA
  P_DIN_DMA_InitDMA();
  // init vom NVIC
  P_DIN_DMA_InitNVIC();
}


//--------------------------------------------------------------
// start vom einlesen per Timer und DMA
//--------------------------------------------------------------
void UB_DIN_DMA_Start(void)
{
  if(din_dma_status==DIN_DMA_READY) {
    din_dma_status=DIN_DMA_RUNNING;
    // timer starten
    P_DIN_DMA_start();
  }
}


//--------------------------------------------------------------
// check ob das einlesen fertig ist
//
// ret_wert :
//   DIN_DMA_READY    = einlesen ist fertig
//   DIN_DMA_RUNNING  = einlesen luft noch
//--------------------------------------------------------------
DIN_DMA_STATUS_t UB_DIN_DMA_Check(void)
{
  return(din_dma_status);
}


//--------------------------------------------------------------
// liesst einen Digital-Wert von einen Kanal, an einer Position aus
// kanal      : [0 bis 7]  bei 8 Kanlen
//              [0 bis 15] bei 16 Kanlen
// position   : [0 bis DIN_DMA_LENGTH-1]
//
// ret_wert   : [0 oder 1]
//--------------------------------------------------------------
uint8_t UB_DIN_DMA_ReadKanal(uint8_t kanal, uint32_t position)
{
  uint8_t ret_wert=0;
  uint16_t bit_maske,wert;

  if(din_dma_status==DIN_DMA_RUNNING) return(0);

  if(position>=DIN_DMA_LENGTH) position=(DIN_DMA_LENGTH-1);

  // Puffer-Werte auslesen
  #ifdef DIN_DMA_USE_BIT0TO15
    // 16 Kanle
    if(kanal>15) kanal=15;

    // wert an position auslesen
    wert=DIN_DMA_PUFFER[position];

    // digital wert bestimmen
    bit_maske=(1<<kanal);
    if((wert&bit_maske)!=0) ret_wert=1;

  #else
    // 8 Kanle
    if(kanal>7) kanal=7;

    // wert an position auslesen
    wert=DIN_DMA_PUFFER[position];

    // digital wert bestimmen
    bit_maske=(1<<kanal);
    if((wert&bit_maske)!=0) ret_wert=1;

  #endif

  return(ret_wert);
}


//--------------------------------------------------------------
// liesst den Wert aller Kanle, an einer Position aus
// position   : [0 bis DIN_DMA_LENGTH-1]
//
// ret_wert   : [0 bis 255]   bei 8 Kanlen
//              [0 bis 65535] bei 16 Kanlen
//--------------------------------------------------------------
uint16_t UB_DIN_DMA_ReadAll(uint32_t position)
{
  uint16_t ret_wert=0;

  if(din_dma_status==DIN_DMA_RUNNING) return(0);

  if(position>=DIN_DMA_LENGTH) position=(DIN_DMA_LENGTH-1);

  // wert an position auslesen
  ret_wert=DIN_DMA_PUFFER[position];

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktionen
// init aller IO-Pins
//--------------------------------------------------------------
void P_DIN_DMA_InitIO(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;

  //---------------------------------------------
  // init aller Pins
  // als normale GPIOs Eingaenge (mit PullUp)
  //---------------------------------------------

  // Clock Enable
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);

  // Config als Digital-Eingang
  #ifdef DIN_DMA_USE_BIT0TO7
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
      GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
  #elif DIN_DMA_USE_BIT8TO15
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 |
        GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  #else
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
      GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7 |
      GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 |
      GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  #endif
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOE, &GPIO_InitStructure);
}


//--------------------------------------------------------------
// interne Funktionen
// init vom Timer
//--------------------------------------------------------------
void P_DIN_DMA_InitTIM(void)
{
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

  //---------------------------------------------
  // init vom Timer1
  // fr GPIO-Daten per DMA
  //---------------------------------------------

  // Clock enable
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);

  // Timer1 init
  TIM_TimeBaseStructure.TIM_Period =  DIN_DMA_TIM1_PERIODE;
  TIM_TimeBaseStructure.TIM_Prescaler = DIN_DMA_TIM1_PRESCALE;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);
}


//--------------------------------------------------------------
// interne Funktionen
// init vom DMA
//--------------------------------------------------------------
void P_DIN_DMA_InitDMA(void)
{
  DMA_InitTypeDef DMA_InitStructure;

  //---------------------------------------------
  // DMA per Timer1 Update
  // (siehe Seite 217 vom Referenz Manual)
  // DMA=2, Channel=6, Stream=5
  //---------------------------------------------

  // Clock Enable (DMA)
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

  // DMA init (DMA2, Channel6, Stream5)
  DMA_Cmd(DMA2_Stream5, DISABLE);
  DMA_DeInit(DMA2_Stream5);
  DMA_InitStructure.DMA_Channel = DMA_Channel_6;
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)DIN_DMA_GPIOE_IDR_ADDRESS;
  DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&DIN_DMA_PUFFER;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
  DMA_InitStructure.DMA_BufferSize = DIN_DMA_LENGTH;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  #ifdef DIN_DMA_USE_BIT0TO15
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;  // 16bit
    DMA_InitStructure.DMA_MemoryDataSize = DMA_PeripheralDataSize_HalfWord;
  #else
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte; // 8bit
    DMA_InitStructure.DMA_MemoryDataSize = DMA_PeripheralDataSize_Byte;
  #endif
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
  DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
  DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_HalfFull;
  DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
  DMA_Init(DMA2_Stream5, &DMA_InitStructure);

  // DMA enable
  DMA_Cmd(DMA2_Stream5, ENABLE);
}


//--------------------------------------------------------------
// interne Funktion
// init vom NVIC
//--------------------------------------------------------------
void P_DIN_DMA_InitNVIC(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  //---------------------------------------------
  // init vom DMA Interrupt
  // fr TransferComplete Interrupt
  // DMA2, Stream5, Channel6
  //---------------------------------------------

  DMA_ITConfig(DMA2_Stream5, DMA_IT_TC, ENABLE);

  // NVIC konfig
  NVIC_InitStructure.NVIC_IRQChannel = DMA2_Stream5_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}


//--------------------------------------------------------------
// interne Funktionen
// start vom Timer
//--------------------------------------------------------------
void P_DIN_DMA_start(void)
{
  // DMA-Timer1 enable
  TIM_DMACmd(TIM1,TIM_DMA_Update,ENABLE);

  // Timer1 enable
  TIM_ARRPreloadConfig(TIM1, ENABLE);
  TIM_Cmd(TIM1, ENABLE);
}


//--------------------------------------------------------------
// Interrupt (ISR-Funktion)
// wird bei DMA TransferCompleteInterrupt aufgerufen
// (DMA2, Stream5, Channel6)
//--------------------------------------------------------------
void DMA2_Stream5_IRQHandler(void)
{
  if(DMA_GetITStatus(DMA2_Stream5, DMA_IT_TCIF5))
  {
    // TransferInterruptComplete Interrupt von DMA2 ist aufgetreten
    DMA_ClearITPendingBit(DMA2_Stream5, DMA_IT_TCIF5);

    // timer1 stoppen
    TIM_Cmd(TIM1, DISABLE);
    din_dma_status=DIN_DMA_READY;
  }  
}
